function noop() {}

function run(fn) {
	return fn();
}

function blank_object() {
	return Object.create(null);
}

function run_all(fns) {
	fns.forEach(run);
}

function is_function(thing) {
	return typeof thing === 'function';
}

function safe_not_equal(a, b) {
	return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
}

function append(target, node) {
	target.appendChild(node);
}

function insert(target, node, anchor) {
	target.insertBefore(node, anchor);
}

function detach(node) {
	node.parentNode.removeChild(node);
}

function element(name) {
	return document.createElement(name);
}

function text(data) {
	return document.createTextNode(data);
}

function space() {
	return text(' ');
}

function listen(node, event, handler, options) {
	node.addEventListener(event, handler, options);
	return () => node.removeEventListener(event, handler, options);
}

function attr(node, attribute, value) {
	if (value == null) node.removeAttribute(attribute);
	else node.setAttribute(attribute, value);
}

function children(element) {
	return Array.from(element.childNodes);
}

function set_style(node, key, value) {
	node.style.setProperty(key, value);
}

function toggle_class(element, name, toggle) {
	element.classList[toggle ? 'add' : 'remove'](name);
}

function custom_event(type, detail) {
	const e = document.createEvent('CustomEvent');
	e.initCustomEvent(type, false, false, detail);
	return e;
}

let current_component;

function set_current_component(component) {
	current_component = component;
}

function get_current_component() {
	if (!current_component) throw new Error(`Function called outside component initialization`);
	return current_component;
}

function onMount(fn) {
	get_current_component().$$.on_mount.push(fn);
}

function onDestroy(fn) {
	get_current_component().$$.on_destroy.push(fn);
}

function createEventDispatcher() {
	const component = current_component;

	return (type, detail) => {
		const callbacks = component.$$.callbacks[type];

		if (callbacks) {
			// TODO are there situations where events could be dispatched
			// in a server (non-DOM) environment?
			const event = custom_event(type, detail);
			callbacks.slice().forEach(fn => {
				fn.call(component, event);
			});
		}
	};
}

const dirty_components = [];

let update_promise;
const binding_callbacks = [];
const render_callbacks = [];
const flush_callbacks = [];

function schedule_update() {
	if (!update_promise) {
		update_promise = Promise.resolve();
		update_promise.then(flush);
	}
}

function add_binding_callback(fn) {
	binding_callbacks.push(fn);
}

function add_render_callback(fn) {
	render_callbacks.push(fn);
}

function flush() {
	const seen_callbacks = new Set();

	do {
		// first, call beforeUpdate functions
		// and update components
		while (dirty_components.length) {
			const component = dirty_components.shift();
			set_current_component(component);
			update(component.$$);
		}

		while (binding_callbacks.length) binding_callbacks.shift()();

		// then, once components are updated, call
		// afterUpdate functions. This may cause
		// subsequent updates...
		while (render_callbacks.length) {
			const callback = render_callbacks.pop();
			if (!seen_callbacks.has(callback)) {
				callback();

				// ...so guard against infinite loops
				seen_callbacks.add(callback);
			}
		}
	} while (dirty_components.length);

	while (flush_callbacks.length) {
		flush_callbacks.pop()();
	}

	update_promise = null;
}

function update($$) {
	if ($$.fragment) {
		$$.update($$.dirty);
		run_all($$.before_render);
		$$.fragment.p($$.dirty, $$.ctx);
		$$.dirty = null;

		$$.after_render.forEach(add_render_callback);
	}
}

function mount_component(component, target, anchor) {
	const { fragment, on_mount, on_destroy, after_render } = component.$$;

	fragment.m(target, anchor);

	// onMount happens after the initial afterUpdate. Because
	// afterUpdate callbacks happen in reverse order (inner first)
	// we schedule onMount callbacks before afterUpdate callbacks
	add_render_callback(() => {
		const new_on_destroy = on_mount.map(run).filter(is_function);
		if (on_destroy) {
			on_destroy.push(...new_on_destroy);
		} else {
			// Edge case - component was destroyed immediately,
			// most likely as a result of a binding initialising
			run_all(new_on_destroy);
		}
		component.$$.on_mount = [];
	});

	after_render.forEach(add_render_callback);
}

function destroy(component, detaching) {
	if (component.$$) {
		run_all(component.$$.on_destroy);
		component.$$.fragment.d(detaching);

		// TODO null out other refs, including component.$$ (but need to
		// preserve final state?)
		component.$$.on_destroy = component.$$.fragment = null;
		component.$$.ctx = {};
	}
}

function make_dirty(component, key) {
	if (!component.$$.dirty) {
		dirty_components.push(component);
		schedule_update();
		component.$$.dirty = {};
	}
	component.$$.dirty[key] = true;
}

function init(component, options, instance, create_fragment, not_equal$$1, prop_names) {
	const parent_component = current_component;
	set_current_component(component);

	const props = options.props || {};

	const $$ = component.$$ = {
		fragment: null,
		ctx: null,

		// state
		props: prop_names,
		update: noop,
		not_equal: not_equal$$1,
		bound: blank_object(),

		// lifecycle
		on_mount: [],
		on_destroy: [],
		before_render: [],
		after_render: [],
		context: new Map(parent_component ? parent_component.$$.context : []),

		// everything else
		callbacks: blank_object(),
		dirty: null
	};

	let ready = false;

	$$.ctx = instance
		? instance(component, props, (key, value) => {
			if ($$.ctx && not_equal$$1($$.ctx[key], $$.ctx[key] = value)) {
				if ($$.bound[key]) $$.bound[key](value);
				if (ready) make_dirty(component, key);
			}
		})
		: props;

	$$.update();
	ready = true;
	run_all($$.before_render);
	$$.fragment = create_fragment($$.ctx);

	if (options.target) {
		if (options.hydrate) {
			$$.fragment.l(children(options.target));
		} else {
			$$.fragment.c();
		}

		if (options.intro && component.$$.fragment.i) component.$$.fragment.i();
		mount_component(component, options.target, options.anchor);
		flush();
	}

	set_current_component(parent_component);
}

class SvelteComponent {
	$destroy() {
		destroy(this, true);
		this.$destroy = noop;
	}

	$on(type, callback) {
		const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
		callbacks.push(callback);

		return () => {
			const index = callbacks.indexOf(callback);
			if (index !== -1) callbacks.splice(index, 1);
		};
	}

	$set() {
		// overridden by instance, if it has props
	}
}

/**
 * Compute the dimension of the crop area based on image size and aspect ratio
 * @param {number} imgWidth width of the src image in pixels
 * @param {number} imgHeight height of the src image in pixels
 * @param {number} aspect aspect ratio of the crop
 */
function getCropSize(imgWidth, imgHeight, aspect) {
  if (imgWidth >= imgHeight * aspect) {
    return {
      width: imgHeight * aspect,
      height: imgHeight,
    }
  }
  return {
    width: imgWidth,
    height: imgWidth / aspect,
  }
}

/**
 * Ensure a new image position stays in the crop area.
 * @param {{x: number, y number}} position new x/y position requested for the image
 * @param {{width: number, height: number}} imageSize width/height of the src image
 * @param {{width: number, height: number}} cropSize width/height of the crop area
 * @param {number} zoom zoom value
 * @returns {{x: number, y number}}
 */
function restrictPosition(position, imageSize, cropSize, zoom) {
  return {
    x: restrictPositionCoord(position.x, imageSize.width, cropSize.width, zoom),
    y: restrictPositionCoord(position.y, imageSize.height, cropSize.height, zoom),
  }
}

function restrictPositionCoord(position, imageSize, cropSize, zoom) {
  const maxPosition = (imageSize * zoom) / 2 - cropSize / 2;
  return Math.min(maxPosition, Math.max(position, -maxPosition))
}

function getDistanceBetweenPoints(pointA, pointB) {
  return Math.sqrt(Math.pow(pointA.y - pointB.y, 2) + Math.pow(pointA.x - pointB.x, 2))
}

/**
 * Compute the output cropped area of the image in percentages and pixels.
 * x/y are the top-left coordinates on the src image
 * @param {{x: number, y number}} crop x/y position of the current center of the image
 * @param {{width: number, height: number, naturalWidth: number, naturelHeight: number}} imageSize width/height of the src image (default is size on the screen, natural is the original size)
 * @param {{width: number, height: number}} cropSize width/height of the crop area
 * @param {number} aspect aspect value
 * @param {number} zoom zoom value
 * @param {boolean} restrictPosition whether we should limit or not the cropped area
 */
function computeCroppedArea(crop, imgSize, cropSize, aspect, zoom, restrictPosition = true) {
  const limitAreaFn = restrictPosition ? limitArea : noOp;
  const croppedAreaPercentages = {
    x: limitAreaFn(
      100,
      (((imgSize.width - cropSize.width / zoom) / 2 - crop.x / zoom) / imgSize.width) * 100
    ),
    y: limitAreaFn(
      100,
      (((imgSize.height - cropSize.height / zoom) / 2 - crop.y / zoom) / imgSize.height) * 100
    ),
    width: limitAreaFn(100, ((cropSize.width / imgSize.width) * 100) / zoom),
    height: limitAreaFn(100, ((cropSize.height / imgSize.height) * 100) / zoom),
  };

  // we compute the pixels size naively
  const widthInPixels = limitAreaFn(
    imgSize.naturalWidth,
    (croppedAreaPercentages.width * imgSize.naturalWidth) / 100,
    true
  );
  const heightInPixels = limitAreaFn(
    imgSize.naturalHeight,
    (croppedAreaPercentages.height * imgSize.naturalHeight) / 100,
    true
  );
  const isImgWiderThanHigh = imgSize.naturalWidth >= imgSize.naturalHeight * aspect;

  // then we ensure the width and height exactly match the aspect (to avoid rounding approximations)
  // if the image is wider than high, when zoom is 0, the crop height will be equals to iamge height
  // thus we want to compute the width from the height and aspect for accuracy.
  // Otherwise, we compute the height from width and aspect.
  const sizePixels = isImgWiderThanHigh
    ? {
        width: Math.round(heightInPixels * aspect),
        height: heightInPixels,
      }
    : {
        width: widthInPixels,
        height: Math.round(widthInPixels / aspect),
      };
  const croppedAreaPixels = {
    ...sizePixels,
    x: limitAreaFn(
      imgSize.naturalWidth - sizePixels.width,
      (croppedAreaPercentages.x * imgSize.naturalWidth) / 100,
      true
    ),
    y: limitAreaFn(
      imgSize.naturalHeight - sizePixels.height,
      (croppedAreaPercentages.y * imgSize.naturalHeight) / 100,
      true
    ),
  };
  return { croppedAreaPercentages, croppedAreaPixels }
}

/**
 * Ensure the returned value is between 0 and max
 * @param {number} max
 * @param {number} value
 * @param {boolean} shouldRound
 */
function limitArea(max, value, shouldRound = false) {
  const v = shouldRound ? Math.round(value) : value;
  return Math.min(max, Math.max(0, v))
}

function noOp(max, value) {
  return value
}

/**
 * Return the point that is the center of point a and b
 * @param {{x: number, y: number}} a
 * @param {{x: number, y: number}} b
 */
function getCenter(a, b) {
  return {
    x: (b.x + a.x) / 2,
    y: (b.y + a.y) / 2,
  }
}

var helpers = /*#__PURE__*/Object.freeze({
	getCropSize: getCropSize,
	restrictPosition: restrictPosition,
	getDistanceBetweenPoints: getDistanceBetweenPoints,
	computeCroppedArea: computeCroppedArea,
	getCenter: getCenter
});

/* src/Cropper.svelte generated by Svelte v3.0.0 */

function add_css() {
	var style = element("style");
	style.id = 'svelte-ohddjv-style';
	style.textContent = ".container.svelte-ohddjv{position:absolute;top:0;left:0;right:0;bottom:0;overflow:hidden;user-select:none;touch-action:none;cursor:move}.image.svelte-ohddjv{max-width:100%;max-height:100%;margin:auto;position:absolute;top:0;bottom:0;left:0;right:0;willchange:transform}.cropperArea.svelte-ohddjv{position:absolute;left:50%;top:50%;transform:translate(-50%, -50%);box-shadow:0 0 0 9999em;box-sizing:border-box;color:rgba(0, 0, 0, 0.5);border:1px solid rgba(255, 255, 255, 0.5);overflow:hidden}.grid.svelte-ohddjv:before{content:' ';box-sizing:border-box;border:1px solid rgba(255, 255, 255, 0.5);position:absolute;top:0;bottom:0;left:33.33%;right:33.33%;border-top:0;border-bottom:0}.grid.svelte-ohddjv:after{content:' ';box-sizing:border-box;border:1px solid rgba(255, 255, 255, 0.5);position:absolute;top:33.33%;bottom:33.33%;left:0;right:0;border-left:0;border-right:0}.round.svelte-ohddjv{border-radius:50%}";
	append(document.head, style);
}

// (329:1) {#if cropperSize}
function create_if_block(ctx) {
	var div;

	return {
		c() {
			div = element("div");
			div.className = "cropperArea svelte-ohddjv";
			set_style(div, "width", "" + ctx.cropperSize.width + "px");
			set_style(div, "height", "" + ctx.cropperSize.height + "px");
			div.dataset.testid = "cropper";
			toggle_class(div, "round", ctx.cropShape === 'round');
			toggle_class(div, "grid", ctx.showGrid);
		},

		m(target, anchor) {
			insert(target, div, anchor);
		},

		p(changed, ctx) {
			if (changed.cropperSize) {
				set_style(div, "width", "" + ctx.cropperSize.width + "px");
				set_style(div, "height", "" + ctx.cropperSize.height + "px");
			}

			if (changed.cropShape) {
				toggle_class(div, "round", ctx.cropShape === 'round');
			}

			if (changed.showGrid) {
				toggle_class(div, "grid", ctx.showGrid);
			}
		},

		d(detaching) {
			if (detaching) {
				detach(div);
			}
		}
	};
}

function create_fragment(ctx) {
	var div, img, t, dispose;

	var if_block = (ctx.cropperSize) && create_if_block(ctx);

	return {
		c() {
			div = element("div");
			img = element("img");
			t = space();
			if (if_block) if_block.c();
			img.className = "image svelte-ohddjv";
			img.src = ctx.image;
			img.alt = "";
			set_style(img, "transform", "translate(" + ctx.crop.x + "px, " + ctx.crop.y + "px) scale(" + ctx.zoom + ")");
			attr(img, "crossorigin", ctx.crossOrigin);
			div.className = "container svelte-ohddjv";
			div.dataset.testid = "container";

			dispose = [
				listen(img, "load", ctx.onImgLoad),
				listen(div, "mousedown", ctx.onMouseDown),
				listen(div, "touchstart", ctx.onTouchStart)
			];
		},

		m(target, anchor) {
			insert(target, div, anchor);
			append(div, img);
			add_binding_callback(() => ctx.img_binding(img, null));
			append(div, t);
			if (if_block) if_block.m(div, null);
			add_binding_callback(() => ctx.div_binding(div, null));
		},

		p(changed, ctx) {
			if (changed.items) {
				ctx.img_binding(null, img);
				ctx.img_binding(img, null);
			}

			if (changed.image) {
				img.src = ctx.image;
			}

			if (changed.crop || changed.zoom) {
				set_style(img, "transform", "translate(" + ctx.crop.x + "px, " + ctx.crop.y + "px) scale(" + ctx.zoom + ")");
			}

			if (changed.crossOrigin) {
				attr(img, "crossorigin", ctx.crossOrigin);
			}

			if (ctx.cropperSize) {
				if (if_block) {
					if_block.p(changed, ctx);
				} else {
					if_block = create_if_block(ctx);
					if_block.c();
					if_block.m(div, null);
				}
			} else if (if_block) {
				if_block.d(1);
				if_block = null;
			}

			if (changed.items) {
				ctx.div_binding(null, div);
				ctx.div_binding(div, null);
			}
		},

		i: noop,
		o: noop,

		d(detaching) {
			if (detaching) {
				detach(div);
			}

			ctx.img_binding(null, img);
			if (if_block) if_block.d();
			ctx.div_binding(null, div);
			run_all(dispose);
		}
	};
}

function instance($$self, $$props, $$invalidate) {
	

	let { image, crop = { x: 0, y: 0 }, zoom = 1, aspect = 4 / 3, minZoom = 1, maxZoom = 3, cropSize = null, cropShape = 'rect', showGrid = true, zoomSpeed = 1, crossOrigin = null, restrictPosition: restrictPosition$1 = true } = $$props;

	let cropperSize = null;
	let imageSize = { width: 0, height: 0, naturalWidth: 0, naturalHeight: 0 };
	let containerEl = null;
	let containerRect = {};
	let imgEl = null;
	let dragStartPosition = { x: 0, y: 0 };
	let dragStartCrop = { x: 0, y: 0 };
	let lastPinchDistance = 0;
	let rafDragTimeout = null;
	let rafZoomTimeout = null;

	const dispatch = createEventDispatcher();

	onMount(() => {
		window.addEventListener('resize', computeSizes);
		containerEl.addEventListener('wheel', onWheel, { passive: false });
		containerEl.addEventListener('gesturestart', preventZoomSafari);
		containerEl.addEventListener('gesturechange', preventZoomSafari);

		// when rendered via SSR, the image can already be loaded and its onLoad callback will never be called
		if (imgEl && imgEl.complete) {
			onImgLoad();
		}
	});

	onDestroy(() => {
		window.removeEventListener('resize', computeSizes);
		containerEl.removeEventListener('wheel', onWheel);
		containerEl.removeEventListener('gesturestart', preventZoomSafari);
		containerEl.removeEventListener('gesturechange', preventZoomSafari);
		cleanEvents();
	});

	// this is to prevent Safari on iOS >= 10 to zoom the page
	const preventZoomSafari = e => e.preventDefault();

	const cleanEvents = () => {
		document.removeEventListener('mousemove', onMouseMove);
		document.removeEventListener('mouseup', onDragStopped);
		document.removeEventListener('touchmove', onTouchMove);
		document.removeEventListener('touchend', onDragStopped);
	};

	const onImgLoad = () => {
		computeSizes();
		emitCropData();
	};

	const getAspect = () => {
		if (cropSize) {
			return cropSize.width / cropSize.height
		}
		return aspect
	};

	const computeSizes = () => {
		if (imgEl) {
			$$invalidate('imageSize', imageSize = {
				width: imgEl.width,
				height: imgEl.height,
				naturalWidth: imgEl.naturalWidth,
				naturalHeight: imgEl.naturalHeight,
			});
			$$invalidate('cropperSize', cropperSize = cropSize 
				? cropSize
				: getCropSize(imgEl.width, imgEl.height, aspect));
		}
		if (containerEl) {
			$$invalidate('containerRect', containerRect = containerEl.getBoundingClientRect());
		}
	};

	const getMousePoint = e => ({ x: Number(e.clientX), y: Number(e.clientY) });

	const getTouchPoint = touch => ({
		x: Number(touch.clientX),
		y: Number(touch.clientY),
	});

	const onMouseDown = e => {
		e.preventDefault();
		document.addEventListener('mousemove', onMouseMove);
		document.addEventListener('mouseup', onDragStopped);
		onDragStart(getMousePoint(e));
	};

	const onMouseMove = e => onDrag(getMousePoint(e));

	const onTouchStart = e => {
		e.preventDefault();
		document.addEventListener('touchmove', onTouchMove, { passive: false }); // iOS 11 now defaults to passive: true
		document.addEventListener('touchend', onDragStopped);
		if (e.touches.length === 2) {
			onPinchStart(e);
		} else if (e.touches.length === 1) {
			onDragStart(getTouchPoint(e.touches[0]));
		}
	};

	const onTouchMove = e => {
		// Prevent whole page from scrolling on iOS.
		e.preventDefault();
		if (e.touches.length === 2) {
			onPinchMove(e);
		} else if (e.touches.length === 1) {
			onDrag(getTouchPoint(e.touches[0]));
		}
	};

	const onDragStart = ({ x, y }) => {
		$$invalidate('dragStartPosition', dragStartPosition = { x, y });
		$$invalidate('dragStartCrop', dragStartCrop = { x: crop.x, y: crop.y });
	};

	const onDrag = ({ x, y }) => {
		if (rafDragTimeout) window.cancelAnimationFrame(rafDragTimeout);

		$$invalidate('rafDragTimeout', rafDragTimeout = window.requestAnimationFrame(() => {
			if (x === undefined || y === undefined) return
			const offsetX = x - dragStartPosition.x;
			const offsetY = y - dragStartPosition.y;
			const requestedPosition = {
				x: dragStartCrop.x + offsetX,
				y: dragStartCrop.y + offsetY,
			};

			$$invalidate('crop', crop = restrictPosition$1
				? restrictPosition(requestedPosition, imageSize, cropperSize, zoom)
				: requestedPosition);
		}));
	};

	const onDragStopped = () => {
		cleanEvents();
		emitCropData();
	};

	const onPinchStart = e => {
		const pointA = getTouchPoint(e.touches[0]);
		const pointB = getTouchPoint(e.touches[1]);
		$$invalidate('lastPinchDistance', lastPinchDistance = getDistanceBetweenPoints(pointA, pointB));
		onDragStart(getCenter(pointA, pointB));
	};

	const onPinchMove = e => {
		const pointA = getTouchPoint(e.touches[0]);
		const pointB = getTouchPoint(e.touches[1]);
		const center = getCenter(pointA, pointB);
		onDrag(center);

		if (rafZoomTimeout) window.cancelAnimationFrame(rafZoomTimeout);
		$$invalidate('rafZoomTimeout', rafZoomTimeout = window.requestAnimationFrame(() => {
			const distance = getDistanceBetweenPoints(pointA, pointB);
			const newZoom = zoom * (distance / lastPinchDistance);
			setNewZoom(newZoom, center);
			$$invalidate('lastPinchDistance', lastPinchDistance = distance);
		}));
	};

	const onWheel = e => {
		e.preventDefault();
		const point = getMousePoint(e);
		const newZoom = zoom - (e.deltaY * zoomSpeed) / 200;
		setNewZoom(newZoom, point);
	};

	const getPointOnContainer = ({ x, y }) => {
		if (!containerRect) {
			throw new Error('The Cropper is not mounted')
		}
		return {
			x: containerRect.width / 2 - (x - containerRect.left),
			y: containerRect.height / 2 - (y - containerRect.top),
		}
	};

	const getPointOnImage = ({ x, y }) => ({
		x: (x + crop.x) / zoom,
		y: (y + crop.y) / zoom,
	});

	const setNewZoom = (newZoom, point) => {
		const zoomPoint = getPointOnContainer(point);
		const zoomTarget = getPointOnImage(zoomPoint);
		$$invalidate('zoom', zoom = Math.min(maxZoom, Math.max(newZoom, minZoom)));

		const requestedPosition = {
			x: zoomTarget.x * zoom - zoomPoint.x,
			y: zoomTarget.y * zoom - zoomPoint.y,
		};
		$$invalidate('crop', crop = restrictPosition$1
			? restrictPosition(requestedPosition, imageSize, cropperSize, zoom)
			: requestedPosition);
	};

	const emitCropData = () => {
		if (!cropperSize || cropperSize.width === 0) return
		// this is to ensure the crop is correctly restricted after a zoom back (https://github.com/ricardo-ch/svelte-easy-crop/issues/6)
		const position = restrictPosition$1
			? restrictPosition(crop, imageSize, cropperSize, zoom)
			: crop;
		const { croppedAreaPercentages, croppedAreaPixels } = computeCroppedArea(
			position,
			imageSize,
			cropperSize,
			getAspect(),
			zoom,
			restrictPosition$1
		);

		dispatch('cropcomplete', {
			percent: croppedAreaPercentages,
			pixels: croppedAreaPixels,
		});
	};

	function img_binding($$node, check) {
		imgEl = $$node;
		$$invalidate('imgEl', imgEl);
	}

	function div_binding($$node, check) {
		containerEl = $$node;
		$$invalidate('containerEl', containerEl);
	}

	$$self.$set = $$props => {
		if ('image' in $$props) $$invalidate('image', image = $$props.image);
		if ('crop' in $$props) $$invalidate('crop', crop = $$props.crop);
		if ('zoom' in $$props) $$invalidate('zoom', zoom = $$props.zoom);
		if ('aspect' in $$props) $$invalidate('aspect', aspect = $$props.aspect);
		if ('minZoom' in $$props) $$invalidate('minZoom', minZoom = $$props.minZoom);
		if ('maxZoom' in $$props) $$invalidate('maxZoom', maxZoom = $$props.maxZoom);
		if ('cropSize' in $$props) $$invalidate('cropSize', cropSize = $$props.cropSize);
		if ('cropShape' in $$props) $$invalidate('cropShape', cropShape = $$props.cropShape);
		if ('showGrid' in $$props) $$invalidate('showGrid', showGrid = $$props.showGrid);
		if ('zoomSpeed' in $$props) $$invalidate('zoomSpeed', zoomSpeed = $$props.zoomSpeed);
		if ('crossOrigin' in $$props) $$invalidate('crossOrigin', crossOrigin = $$props.crossOrigin);
		if ('restrictPosition' in $$props) $$invalidate('restrictPosition', restrictPosition$1 = $$props.restrictPosition);
	};

	$$self.$$.update = ($$dirty = { imgEl: 1, cropSize: 1, aspect: 1, zoom: 1 }) => {
		if ($$dirty.imgEl || $$dirty.cropSize || $$dirty.aspect) { if (imgEl) {
				$$invalidate('cropperSize', cropperSize = cropSize 
						? cropSize
						: getCropSize(imgEl.width, imgEl.height, aspect));
			} }
		if ($$dirty.zoom) { zoom && emitCropData(); }
	};

	return {
		image,
		crop,
		zoom,
		aspect,
		minZoom,
		maxZoom,
		cropSize,
		cropShape,
		showGrid,
		zoomSpeed,
		crossOrigin,
		restrictPosition: restrictPosition$1,
		cropperSize,
		containerEl,
		imgEl,
		onImgLoad,
		onMouseDown,
		onTouchStart,
		img_binding,
		div_binding
	};
}

class Cropper extends SvelteComponent {
	constructor(options) {
		super();
		if (!document.getElementById("svelte-ohddjv-style")) add_css();
		init(this, options, instance, create_fragment, safe_not_equal, ["image", "crop", "zoom", "aspect", "minZoom", "maxZoom", "cropSize", "cropShape", "showGrid", "zoomSpeed", "crossOrigin", "restrictPosition"]);
	}
}

export default Cropper;
